/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef atomic_h
#define atomic_h

#if defined(__clang__)

#define AtomicInt int

#define atomic_int_inc(a) __sync_fetch_and_add((a),1)
#define atomic_int_dec(a) __sync_fetch_and_sub((a),1)
#define atomic_int_test_and_inc(a) (__sync_fetch_and_add((a),1)<=0)
#define atomic_int_dec_and_test(a) (__sync_fetch_and_sub((a),1)<=1)

#elif defined(__GCC_ATOMIC_INT_LOCK_FREE)

#define AtomicInt int

#define atomic_int_inc(a) __sync_fetch_and_add((a),1)
#define atomic_int_dec(a) __sync_fetch_and_sub((a),1)
#define atomic_int_test_and_inc(a) (__sync_fetch_and_add((a),1)<=0)
#define atomic_int_dec_and_test(a) (__sync_fetch_and_sub((a),1)<=1)

#elif defined(USE_ATOMIC_OPS)

#include <atomic_ops.h>

#define AtomicInt AO_t

#define atomic_int_inc(a) fetch_and_add(a,1)
#define atomic_dec_and_test ...

#elif defined(USE_GLIB)

#include <glib.h>

#define atomic_int_inc(a) g_atomic_int_inc(a)
#define atomic_int_dec(a) g_atomic_int_inc(a)
#define atomic_int_test_and_inc(a) (g_atomic_int_add(a)<=0)
#define atomic_int_dec_and_test(a) g_atomic_int_dec_and_test(a)

#else

/* FIXME!! */

#endif

#endif /* atomic_h */
